// <summary>ソースコード：要素演算データクラス</summary>
// <author>CommonMP</author>

using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Model;
using CommonMP.HYMCO.CoreImpl.Data;

//using CommonMP.HYMCO.OptionImpl.CommonControlLibrary;
//using jp.or.jccajsce.StorageFunctionRiverProperty;

// ToDo namespace は　モデル開発者が変更して、ユニークな名称にして下さい
namespace jp.or.jccajsce.StorageFunctionRiver
{
    /// <summary><para>class outline:</para>
    /// <para>テストサンプル要素内演算中データクラス</para>
    /// </summary>
    /// <remarks><para>history:</para>
    /// <para>[CommonMP][ver 1.0.0][2008/10/01][新規作成]</para>
    /// <para>remarks</para>
    /// <para>演算中の状態、初期条件等を保持するデータクラス</para>
    /// </remarks>
    [Serializable]
    public class StorageFunctionRiverPropertyInfo : McCellModelPropertyInfo
    {

        // ToDo 独自メンバー変数等を追加して下さい。
        //  例
        //   protected double m_dData = 0.0;
        /// <summary> 貯留関数河道データ (Key=日時文字列,Value=SQデータ配列 double[])</summary>
        public SortedList<string, object> m_csSQData = null;


        /// <summary>
        /// SQデータ格納
        /// </summary>
        /// <param name="csData"></param>
        public void SetInfoSQData(SortedList<string, object> csData)
        {
            m_csSQData = csData;
        }

        /// <summary>
        /// SQデータ取得
        /// </summary>
        /// <param name="csData"></param>
        public void GetInfoSQData(ref SortedList<string, object> csData)
        {
            if (m_csSQData == null)
                m_csSQData = new SortedList<string, object>();

            csData = m_csSQData;
        }


        /// <summary><para>method outline:</para>
        /// <para>コンストラクタ</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para>McPropertyInfo csModelPropertyInfoRoot = new MySpecialPropertyData(csLibraryID, csModelKind)</para>
        /// </example>
        /// <param name="csLibraryID">ライブラリー識別子</param>
        /// <param name="csModelKind">モデル種別識別子</param>
        /// <returns>McModelPropertyInfo 生成されたインスタンス</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public StorageFunctionRiverPropertyInfo(HySID csLibraryID, HySObjectKind csModelKind)
            : base(csLibraryID, csModelKind)
        {
            //バージョン情報設定
            SetNowVersion();
        }

        /// <summary><para>method outline:</para>
        /// <para>自分と同じクラスのインスタンスを生成する</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> McPropertyInfoRoot csModelPropertyInfoRoot = Clone( ) </para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>McPropertyInfoRoot 生成されたクラスのインスタンス</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override McPropertyInfoRoot Clone()
        {
            StorageFunctionRiverPropertyInfo csRtn = new StorageFunctionRiverPropertyInfo(m_csLibraryID, m_csModelKind);
            csRtn.CopyInf(this);
            return csRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>引数で与えられた情報を自分にコピーを行う</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> CopyInf(csOrgObj) </para>
        /// </example>
        /// <param name="csOrgObj">コピー元情報</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override void CopyInf(McPropertyInfoRoot csOrgObj)
        {
            base.CopyInf(csOrgObj);
            StorageFunctionRiverPropertyInfo csPrpOrgObj = csOrgObj as StorageFunctionRiverPropertyInfo;

            //SQデータ
            m_csSQData = new SortedList<string, object>();
            if (csPrpOrgObj.m_csSQData != null)
            {
                // ToDo 独自メンバー変数のコピー処理を追加して下さい。
                //    例 this.m_dData = csPrpOrgObj.m_dData 等
                for (int i = 0; i < csPrpOrgObj.m_csSQData.Count; i++)
                {
                    if (csPrpOrgObj.m_csSQData.Values[i] is double[])
                    {
                        //値が1次元
                        double[] dVal = (double[])csPrpOrgObj.m_csSQData.Values[i];
                        double[] dValCopy = new double[dVal.Length];
                        for (int j = 0; j < dVal.Length; j++)
                        {
                            dValCopy[j] = dVal[j];
                        }
                        m_csSQData.Add(csPrpOrgObj.m_csSQData.Keys[i], dValCopy);
                    }
                    else if (csPrpOrgObj.m_csSQData.Values[i] is double[,])
                    {
                        //値が2次元
                    }

                }
            }
        }

        /// <summary><para>method outline:</para>
        /// <para>モデル情報をXMLへ書き出す</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> HySXmlNode csXmlNode = XmlOUT(csXmlWriter) </para>
        /// </example>
        /// <param name="csXmlWriter">XML書き出しクラス</param>
        /// <returns>HySXmlNode XMLノードクラス</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override HySXmlNode XmlOUT(ref HySXmlWriter csXmlWriter)
        {
            HySXmlNode csRtn = base.XmlOUT(ref csXmlWriter);

            // ToDo 独自に追加した変数を　ＸＭＬへ出力する処理を追加してください。
            // 例　
            //   属性として追加する場合
            //   csRtn.SetAttribute("m_dData", m_dData.ToString());

            /* ****************************************** 出力イメージ *************
            <Table Key="m_csSQData" Name="Ｓ－Ｑ関係" AddCols="False" AddRows="True" RowNumHeader="True">
              <Schema>
                <Element Key="m_dSK" Name="パラメータ値Ｋ" DataType="DOUBLE" />
                <Element Key="m_dP" Name="パラメータ値Ｐ" DataType="DOUBLE" />
                <Element Key="m_dQl" Name="適用区分Qo(m3/s)まで" DataType="DOUBLE" />
              </Schema>
              <Row>
                <Col Key="m_dSK" Value="0.03" />
                <Col Key="m_dP" Value="0.03" />
                <Col Key="m_dQl" Value="0.03" />
              </Row>
            </Table>
            ********************************************************************* */

            //データを　ＸＭＬへ出力する
            StructXmlTable csTable1 = new StructXmlTable(csXmlWriter, "m_csSQData", Properties.Resources.STRING_OUT_XML1, false, true, true);
            csTable1.AddSchema("m_dSK", Properties.Resources.STRING_OUT_XML2, McDefine.ValKind.DOUBLE, false);
            csTable1.AddSchema("m_dP", Properties.Resources.STRING_OUT_XML3, McDefine.ValKind.DOUBLE, false);
            csTable1.AddSchema("m_dQl", Properties.Resources.STRING_OUT_XML4, McDefine.ValKind.DOUBLE, false);

            if (m_csSQData != null)
            {
                double[] dSK = (double[])m_csSQData["m_dSK"];
                double[] dP = (double[])m_csSQData["m_dP"];
                double[] dQl = (double[])m_csSQData["m_dQl"];

                for (int iRow = 0; iRow < dSK.Length; iRow++)
                {
                    HySXmlNode csRowNode = csTable1.CreateRow();
                    csTable1.AddCol(csRowNode, "m_dSK", dSK[iRow].ToString());
                    csTable1.AddCol(csRowNode, "m_dP", dP[iRow].ToString());
                    csTable1.AddCol(csRowNode, "m_dQl", dQl[iRow].ToString());
                }
            }
            csRtn.AppendChild(csTable1.m_csTableNode);   // AppendChildは最後に行う

            return csRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>XMLからモデル情報を読み出す</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> bool bRtn = XmlIN(csPrptyXml) </para>
        /// </example>
        /// <param name="csPrptyXml">入力XML</param>
        /// <returns>true :正常 , false:型式不適等</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override bool XmlIN(HySXmlNode csPrptyXml)
        {
            bool bRtn = base.XmlIN(csPrptyXml);

            // ToDo 独自に追加した変数を　ＸＭＬから読み込む処理を追加して下さい。
            // 例　
            //   属性として追加する場合
            //string sValueAttr = "";
            //sValueAttr = csPrptyXml.GetAttribute("m_dData");
            //if (sValueAttr != "")
            //{
            //    m_dData = Convert.ToDouble(sValueAttr);
            //}
            try
            {
                //データを　ＸＭＬから読み込む
                m_csSQData = null;
                m_csSQData = new SortedList<string, object>();
                StructXmlTable csTable1 = new StructXmlTable(csPrptyXml, "m_csSQData");
                double[] dSK = new double[csTable1.m_lRowCnt];
                double[] dP = new double[csTable1.m_lRowCnt];
                double[] dQl = new double[csTable1.m_lRowCnt];
                //変数に追加
                m_csSQData.Add("m_dSK", dSK);
                m_csSQData.Add("m_dP", dP);
                m_csSQData.Add("m_dQl", dQl);

                for (long lRow = 0; lRow < csTable1.m_lRowCnt; lRow++)
                {
                    Dictionary<string, string> csColsList = csTable1.GetCols(lRow);
                    foreach (KeyValuePair<string, string> csCol in csColsList)
                    {
                        double[] dVal = (double[])m_csSQData[csCol.Key];
                        dVal[lRow] = Convert.ToDouble(csCol.Value);
                    }
                }

            }
            catch
            {
                bRtn = false;
            }
            return bRtn;
        }

        // =============================================
        // === バージョンアップ時の互換性を保つ為の処理 ===
        /// <summary>保存バージョン情報</summary>
        private string m_sDecodedVersion;

        /// <summary><para>method outline:</para>
        /// <para>バージョン情報取得</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> string sDecodedVersion = GetDecodedVersion( ) </para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>string 保存バージョン情報</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>メンバー変数に設定されているバージョン情報を返す</para>
        /// </remarks>
        public new string GetDecodedVersion()
        {
            return m_sDecodedVersion;
        }

        /// <summary><para>method outline:</para>
        /// <para>バージョン情報取得</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> string sVersion = GetNowVersion( ) </para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>string 現在のバージョン情報</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public new string GetNowVersion()
        {
            return "1.00";
        }

        /// <summary><para>method outline:</para>
        /// <para>バージョン情報設定</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> SetNowVersion( ) </para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>現在のバージョン情報をメンバー変数に設定する</para>
        /// </remarks>
        public new void SetNowVersion()
        {
            m_sDecodedVersion = GetNowVersion();
        }

        /// <summary><para>method outline:</para>
        /// <para>復元後の整合性確保処理</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> PostDeserialize( ) </para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>シリアライズされた旧バージョン情報を復元する際、過去の整合性をとる時に使用する
        /// バージョンによって変数等追加がない場合には、実装の必要がない
        /// </para>
        /// </remarks>
        public override void PostDeserialize()
        {
            base.PostDeserialize();

            if (GetDecodedVersion() != GetNowVersion())
            {
                //旧バージョンから順にバージョンを上げて行く
                switch (GetDecodedVersion())
                {
                    case "1.00":
                        break;
                }
                SetNowVersion();
            }
        }
    }
}
