﻿// <summary>ソースコード：要素演算データクラス</summary>
// <author>CommonMP</author>

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.Interface.HSViewer;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.Interface.Controller;

// ToDo namespace は　モデル開発者が変更して、ユニークな名称にして下さい
namespace jp.or.jccajsce.StorageFunctionRiver
{
    /// <summary><para>class outline:</para>
    /// <para>テストサンプル要素内演算中データクラス</para>
    /// </summary>
    /// <remarks>
    /// <para>history:</para>
    /// <para>[CommonMP][ver 1.0.0][2008/10/01][新規作成]</para>
    /// <para>remarks</para>
    /// <para>演算中の状態、初期条件等を保持するデータクラス</para>
    /// </remarks>
    [Serializable]
    public class StorageFunctionRiverCalInfo : McCalInfo
    {
        ///// <summary> コーディングイメージです </summary>
        //public double m_dData=-99.9;

        // To Do
        //  必要な情報を追加します。
        public long m_lArrayNum = 0;
        /// <summary>KPテーブル数</summary>
        public long m_lParaNum = 0;
        /// <summary>遅滞時間</summary>
        public double m_dTl = 0;
        /// <summary>ずらし時間</summary>
        public double m_dTlz = 0;
        /// <summary>Ｋ</summary>
        public double[] m_dSc = null;
        /// <summary>Ｐ</summary>
        public double[] m_dPc = null;
        /// <summary>適応流量</summary>
        public double[] m_dQl = null;
        public long m_lDim = 0;
        /// <summary>貯留量</summary>
        public double m_dSS = 0;
        /// <summary>流出量</summary>
        public double m_dQout = 0;
        /// <summary>流出量（ずらし時間考慮）</summary>
        public double m_dQoutTlz = 0;
        /// <summary>Δｔ（hr）</summary>
        public double m_dDT = 0.0;
        /// <summary>前時刻の貯留量</summary>
        public double m_dSSp = 0.0;
        /// <summary>前時刻の流出量</summary>
        public double m_dQoutp = 0.0;

        public SortedList<string, double> m_hashQout = null;

        // 入力情報制御
        /// <summary>上流端入力数</summary>
        public long m_lInDataNum = 0;
        /// <summary>上流端入力のインデックス</summary>
        public long[] m_lInIndx = null;
        /// <summary>流入量</summary>
        public double m_dQin;

        // 出力情報
        /// <summary>下流流量出力数</summary>
        public long m_lOutDataNum;
        /// <summary>出力インデックス</summary>
        public long[] m_lOutIndx = null;
        /// <summary> データを出力した演算時刻 </summary>
        public HySTime m_csLastDataOutTime = null;
        public long m_lOutPutPortNum = 0;

        /// <summary><para>method outline:</para>
        /// <para>自己複製</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> Clone( ) </para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>同一内容で　別インスタンス</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
        /// </remarks>
        public override McCalInfo Clone()
        {
            StorageFunctionRiverCalInfo csRtn = new StorageFunctionRiverCalInfo();
            csRtn.CopyInfo(this);
            return csRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>引数で与えられた情報を自分にコピーを行う</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
        /// </example>
        /// <param name="csOrgInfo">コピー元情報</param>
        /// <returns> bool true :正常 , false:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override bool CopyInfo(McCalInfo csOrgInfo)
        {
            StorageFunctionRiverCalInfo csOrdDt = (StorageFunctionRiverCalInfo)csOrgInfo;

            // this.m_dData = csOrdDt.m_dData; コーディングイメージです

            // To Do
            // 此処に、追加したメンバーをコピーする処理を入れます。
            this.m_lArrayNum = csOrdDt.m_lArrayNum;
            this.m_lParaNum = csOrdDt.m_lParaNum;
            this.m_dTl = csOrdDt.m_dTl;
            this.m_dTlz = csOrdDt.m_dTlz;
            this.m_dSc = new double[this.m_lParaNum];
            this.m_dPc = new double[this.m_lParaNum];
            for (int i = 0; i < this.m_lParaNum; i++)
            {
                this.m_dSc[i] = csOrdDt.m_dSc[i];
                this.m_dPc[i] = csOrdDt.m_dPc[i];
            }
            this.m_dQout = csOrdDt.m_dQout;
            this.m_dQoutTlz = csOrdDt.m_dQoutTlz;
            this.m_lDim = csOrdDt.m_lDim;
            this.m_dSS = csOrdDt.m_dSS;

            this.m_lOutPutPortNum = csOrdDt.m_lOutPutPortNum;
            this.m_csLastDataOutTime = csOrdDt.m_csLastDataOutTime.Clone();
            this.m_dDT = csOrdDt.m_dDT;
            this.m_dSSp = csOrdDt.m_dSSp;
            this.m_dQoutp = csOrdDt.m_dQoutp;
            this.m_hashQout = csOrdDt.m_hashQout;

            return true;
        }

    }
}
